@@ -4,23 +4,17 @@ from __future__ import annotations
 
 import asyncio
 import concurrent.futures as cf
-import datetime
 import io
 import logging
 import pathlib
 import uuid
+from collections.abc import AsyncIterable, AsyncIterator, Awaitable, Iterable, Sequence
 from typing import (
     TYPE_CHECKING,
     Any,
-    AsyncIterable,
-    AsyncIterator,
-    Awaitable,
     Callable,
-    Dict,
-    Iterable,
-    List,
+    Literal,
     Optional,
-    Sequence,
     TypeVar,
     Union,
     cast,
@@ -42,7 +36,7 @@ from langsmith.evaluation._runner import (
     _ExperimentManagerMixin,
     _extract_feedback_keys,
     _ForwardResults,
-    _include_attachments,
+    _get_target_args,
     _is_langchain_runnable,
     _load_examples_map,
     _load_experiment,
@@ -51,6 +45,7 @@ from langsmith.evaluation._runner import (
     _resolve_data,
     _resolve_evaluators,
     _resolve_experiment,
+    _target_include_attachments,
     _to_pandas,
     _wrap_summary_evaluators,
 )
@@ -95,6 +90,7 @@ async def aevaluate(
     blocking: bool = True,
     experiment: Optional[Union[schemas.TracerSession, str, uuid.UUID]] = None,
     upload_results: bool = True,
+    error_handling: Literal["log", "ignore"] = "log",
     **kwargs: Any,
 ) -> AsyncExperimentResults:
     r"""Evaluate an async target system on a given dataset.
@@ -131,6 +127,9 @@ async def aevaluate(
         load_nested: Whether to load all child runs for the experiment.
             Default is to only load the top-level root runs. Should only be specified
             when evaluating an existing experiment.
+        error_handling (str, default="log"): How to handle individual run errors. 'log'
+            will trace the runs with the error message as part of the experiment,
+            'ignore' will not count the run as part of the experiment at all.
 
     Returns:
         AsyncIterator[ExperimentResultRow]: An async iterator over the experiment results.
@@ -335,6 +334,7 @@ async def aevaluate(
             blocking=blocking,
             experiment=experiment,
             upload_results=upload_results,
+            error_handling=error_handling,
         )
 
 
@@ -390,31 +390,44 @@ async def aevaluate_existing(
 
         Load the experiment and run the evaluation.
 
-        >>> from langsmith import aevaluate, aevaluate_existing
-        >>> dataset_name = "Evaluate Examples"
+        >>> import asyncio
+        >>> import uuid
+        >>> from langsmith import Client, aevaluate, aevaluate_existing
+        >>> client = Client()
+        >>> dataset_name = "__doctest_aevaluate_existing_" + uuid.uuid4().hex[:8]
+        >>> dataset = client.create_dataset(dataset_name)
+        >>> example = client.create_example(
+        ...     inputs={"question": "What is 2+2?"},
+        ...     outputs={"answer": "4"},
+        ...     dataset_id=dataset.id,
+        ... )
         >>> async def apredict(inputs: dict) -> dict:
-        ...     # This can be any async function or just an API call to your app.
-        ...     await asyncio.sleep(0.1)
-        ...     return {"output": "Yes"}
-        >>> # First run inference on the dataset
-        ... results = asyncio.run(
+        ...     await asyncio.sleep(0.001)
+        ...     return {"output": "4"}
+        >>> results = asyncio.run(
         ...     aevaluate(
-        ...         apredict,
-        ...         data=dataset_name,
+        ...         apredict, data=dataset_name, experiment_prefix="doctest_experiment"
         ...     )
         ... )  # doctest: +ELLIPSIS
         View the evaluation results for experiment:...
-
-        Then evaluate the results
-        >>> experiment_name = "My Experiment:64e6e91"  # Or manually specify
+        >>> experiment_id = results.experiment_name
+        >>> # Consume all results to ensure evaluation is complete
+        >>> async def consume_results():
+        ...     result_list = [r async for r in results]
+        ...     return len(result_list) > 0
+        >>> asyncio.run(consume_results())
+        True
+        >>> import time
+        >>> time.sleep(3)
         >>> results = asyncio.run(
         ...     aevaluate_existing(
-        ...         experiment_name,
+        ...         experiment_id,
         ...         evaluators=[accuracy],
         ...         summary_evaluators=[precision],
         ...     )
         ... )  # doctest: +ELLIPSIS
         View the evaluation results for experiment:...
+        >>> client.delete_dataset(dataset_id=dataset.id)
 
 
     """  # noqa: E501
@@ -457,6 +470,7 @@ async def _aevaluate(
     blocking: bool = True,
     experiment: Optional[Union[schemas.TracerSession, str, uuid.UUID]] = None,
     upload_results: bool = True,
+    error_handling: Literal["log", "ignore"] = "log",
 ) -> AsyncExperimentResults:
     is_async_target = (
         asyncio.iscoroutinefunction(target)
@@ -471,6 +485,9 @@ async def _aevaluate(
         runs,
         client,
     )
+    num_include_attachments = int(
+        _target_include_attachments(target)
+    ) + _evaluators_include_attachments(evaluators)
     manager = await _AsyncExperimentManager(
         data,
         client=client,
@@ -479,15 +496,10 @@ async def _aevaluate(
         description=description,
         num_repetitions=num_repetitions,
         runs=runs,
-        include_attachments=_include_attachments(target)
-        or _evaluators_include_attachments(evaluators) > 0,
-        reuse_attachments=num_repetitions
-        * (
-            int(_include_attachments(target))
-            + _evaluators_include_attachments(evaluators)
-        )
-        > 1,
+        include_attachments=num_include_attachments > 0,
+        reuse_attachments=num_repetitions * num_include_attachments > 1,
         upload_results=upload_results,
+        error_handling=error_handling,
     ).astart()
     cache_dir = ls_utils.get_cache_dir(None)
     if cache_dir is not None:
@@ -555,6 +567,9 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
         attachment_raw_data_dict (Optional[dict]): A dictionary to store raw data
             for attachments. Only used if we reuse attachments across multiple
             target/evaluator functions.
+        error_handling (str, default="log"): How to handle individual run errors. 'log'
+            will trace the runs with the error message as part of the experiment,
+            'ignore' will not count the run as part of the experiment at all.
     """
 
     def __init__(
@@ -573,6 +588,7 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
         reuse_attachments: bool = False,
         upload_results: bool = True,
         attachment_raw_data_dict: Optional[dict] = None,
+        error_handling: Literal["log", "ignore"] = "log",
     ):
         super().__init__(
             experiment=experiment,
@@ -592,6 +608,7 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
         self._reuse_attachments = reuse_attachments
         self._upload_results = upload_results
         self._attachment_raw_data_dict = attachment_raw_data_dict
+        self._error_handling = error_handling
 
     def _reset_example_attachments(self, example: schemas.Example) -> schemas.Example:
         """Reset attachment readers for an example.
@@ -719,17 +736,9 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
         project = self._get_project(first_example) if self._upload_results else None
         self._print_experiment_start(project, first_example)
         self._metadata["num_repetitions"] = self._num_repetitions
-        return self.__class__(
+        return self._copy(
             await self.aget_examples(),
             experiment=project,
-            metadata=self._metadata,
-            client=self.client,
-            runs=self._runs,
-            evaluation_results=self._evaluation_results,
-            include_attachments=self._include_attachments,
-            reuse_attachments=self._reuse_attachments,
-            upload_results=self._upload_results,
-            attachment_raw_data_dict=self._attachment_raw_data_dict,
         )
 
     def _get_example_with_readers(self, example: schemas.Example) -> schemas.Example:
@@ -779,8 +788,33 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
         """
         evaluators = _resolve_evaluators(evaluators)
 
-        if not hasattr(self, "_evaluator_executor"):
-            self._evaluator_executor = cf.ThreadPoolExecutor(max_workers=4)
+        if not hasattr(self, "_evaluation_feedback_executor"):
+            self._evaluation_feedback_executor = cf.ThreadPoolExecutor(max_workers=4)
+
+        traceable_target = _ensure_async_traceable(target)
+
+        async def process_example(example: schemas.Example):
+            # Yield the coroutine to be awaited later
+            pred = await _aforward(
+                traceable_target,
+                self._get_example_with_readers(example),
+                self.experiment_name,
+                self._metadata,
+                self.client,
+                _target_include_attachments(target),
+                self._error_handling,
+            )
+            example, run = pred["example"], pred["run"]
+            result = await self._arun_evaluators(
+                evaluators,
+                {
+                    "run": run,
+                    "example": example,
+                    "evaluation_results": {"results": []},
+                },
+                feedback_executor=self._evaluation_feedback_executor,
+            )
+            return result
 
         async def process_examples():
             """Create a single task per example.
@@ -788,22 +822,10 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
             That task is to run the target function and all the evaluators
             sequentially.
             """
-            async for pred in self._apredict(
-                target,
-                max_concurrency=max_concurrency,
-                include_attachments=_include_attachments(target),
-            ):
-                example, run = pred["example"], pred["run"]
-                result = self._arun_evaluators(
-                    evaluators,
-                    {
-                        "run": run,
-                        "example": example,
-                        "evaluation_results": {"results": []},
-                    },
-                    executor=self._evaluator_executor,
-                )
-                yield result
+            async for example in await self.aget_examples():
+                yield process_example(example)
+
+            await self._aend()
 
         # Run the per-example tasks with max-concurrency
         # This guarantees that max_concurrency is the upper limit
@@ -816,16 +838,10 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
 
         r1, r2, r3 = aitertools.atee(experiment_results, 3, lock=asyncio.Lock())
 
-        return _AsyncExperimentManager(
+        return self._copy(
             (result["example"] async for result in r1),
-            experiment=self._experiment,
-            metadata=self._metadata,
-            client=self.client,
             runs=(result["run"] async for result in r2),
             evaluation_results=(result["evaluation_results"] async for result in r3),
-            summary_results=self._summary_results,
-            include_attachments=self._include_attachments,
-            upload_results=self._upload_results,
         )
 
     async def awith_predictions(
@@ -837,17 +853,12 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
         _experiment_results = self._apredict(
             target,
             max_concurrency=max_concurrency,
-            include_attachments=_include_attachments(target),
+            include_attachments=_target_include_attachments(target),
         )
         r1, r2 = aitertools.atee(_experiment_results, 2, lock=asyncio.Lock())
-        return _AsyncExperimentManager(
+        return self._copy(
             (pred["example"] async for pred in r1),
-            experiment=self._experiment,
-            metadata=self._metadata,
-            client=self.client,
             runs=(pred["run"] async for pred in r2),
-            include_attachments=self._include_attachments,
-            upload_results=self._upload_results,
         )
 
     async def awith_evaluators(
@@ -859,16 +870,10 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
         evaluators = _resolve_evaluators(evaluators)
         experiment_results = self._ascore(evaluators, max_concurrency=max_concurrency)
         r1, r2, r3 = aitertools.atee(experiment_results, 3, lock=asyncio.Lock())
-        return _AsyncExperimentManager(
+        return self._copy(
             (result["example"] async for result in r1),
-            experiment=self._experiment,
-            metadata=self._metadata,
-            client=self.client,
             runs=(result["run"] async for result in r2),
             evaluation_results=(result["evaluation_results"] async for result in r3),
-            summary_results=self._summary_results,
-            include_attachments=self._include_attachments,
-            upload_results=self._upload_results,
         )
 
     async def awith_summary_evaluators(
@@ -877,16 +882,10 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
     ) -> _AsyncExperimentManager:
         wrapped_evaluators = _wrap_summary_evaluators(summary_evaluators)
         aggregate_feedback_gen = self._aapply_summary_evaluators(wrapped_evaluators)
-        return _AsyncExperimentManager(
+        return self._copy(
             await self.aget_examples(),
-            experiment=self._experiment,
-            metadata=self._metadata,
-            client=self.client,
             runs=self.aget_runs(),
-            evaluation_results=self._evaluation_results,
             summary_results=aggregate_feedback_gen,
-            include_attachments=self._include_attachments,
-            upload_results=self._upload_results,
         )
 
     async def aget_results(self) -> AsyncIterator[ExperimentResultRow]:
@@ -899,7 +898,7 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
                 evaluation_results=evaluation_results,
             )
 
-    async def aget_summary_scores(self) -> Dict[str, List[dict]]:
+    async def aget_summary_scores(self) -> dict[str, list[dict]]:
         if self._summary_results is None:
             return {"results": []}
         return {
@@ -931,6 +930,7 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
                     self._metadata,
                     self.client,
                     include_attachments,
+                    self._error_handling,
                 )
 
         async for result in aitertools.aiter_with_concurrency(
@@ -945,13 +945,13 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
         evaluators: Sequence[RunEvaluator],
         max_concurrency: Optional[int] = None,
     ) -> AsyncIterator[ExperimentResultRow]:
-        with cf.ThreadPoolExecutor(max_workers=4) as executor:
+        with cf.ThreadPoolExecutor(max_workers=4) as feedback_executor:
 
             async def score_all():
                 async for current_results in self.aget_results():
                     # Yield the coroutine to be awaited later in aiter_with_concurrency
                     yield self._arun_evaluators(
-                        evaluators, current_results, executor=executor
+                        evaluators, current_results, feedback_executor=feedback_executor
                     )
 
             async for result in aitertools.aiter_with_concurrency(
@@ -963,7 +963,7 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
         self,
         evaluators: Sequence[RunEvaluator],
         current_results: ExperimentResultRow,
-        executor: cf.ThreadPoolExecutor,
+        feedback_executor: cf.ThreadPoolExecutor,
     ) -> ExperimentResultRow:
         current_context = rh.get_tracing_context()
         metadata = {
@@ -983,11 +983,13 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
             example = current_results["example"]
             eval_results = current_results["evaluation_results"]
 
-            async def _run_single_evaluator(evaluator):
+            async def _run_single_evaluator(evaluator: RunEvaluator):
+                evaluator_run_id = uuid.uuid4()
                 try:
-                    evaluator_response = await evaluator.aevaluate_run(
+                    evaluator_response = await evaluator.aevaluate_run(  # type: ignore[call-arg]
                         run=run,
                         example=self._get_example_with_readers(example),
+                        evaluator_run_id=evaluator_run_id,
                     )
                     selected_results = self.client._select_eval_results(
                         evaluator_response
@@ -995,7 +997,7 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
 
                     if self._upload_results:
                         self.client._log_evaluation_feedback(
-                            evaluator_response, run=run, _executor=executor
+                            evaluator_response, run=run, _executor=feedback_executor
                         )
                     return selected_results
                 except Exception as e:
@@ -1006,7 +1008,7 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
                             results=[
                                 EvaluationResult(
                                     key=key,
-                                    source_run_id=run.id,
+                                    source_run_id=evaluator_run_id,
                                     comment=repr(e),
                                     extra={"error": True},
                                 )
@@ -1018,7 +1020,7 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
                         )
                         if self._upload_results:
                             self.client._log_evaluation_feedback(
-                                error_response, run=run, _executor=executor
+                                error_response, run=run, _executor=feedback_executor
                             )
                         return selected_results
                     except Exception as e2:
@@ -1137,14 +1139,31 @@ class _AsyncExperimentManager(_ExperimentManagerMixin):
         project_metadata["dataset_splits"] = await self._get_dataset_splits()
         self.client.update_project(
             experiment.id,
-            end_time=experiment.end_time
-            or datetime.datetime.now(datetime.timezone.utc),
             metadata={
                 **experiment.metadata,
                 **project_metadata,
             },
         )
 
+    def _copy(self, *args: Any, **kwargs: Any) -> _AsyncExperimentManager:
+        default_args = (self._data,)
+        default_kwargs = {
+            "experiment": self._experiment,
+            "metadata": self._metadata,
+            "runs": self._runs,
+            "client": self.client,
+            "evaluation_results": self._evaluation_results,
+            "summary_results": self._summary_results,
+            "include_attachments": self._include_attachments,
+            "reuse_attachments": self._reuse_attachments,
+            "upload_results": self._upload_results,
+            "attachment_raw_data_dict": self._attachment_raw_data_dict,
+            "error_handling": self._error_handling,
+        }
+        full_args = list(args) + list(default_args[len(args) :])
+        full_kwargs = {**default_kwargs, **kwargs}
+        return self.__class__(*full_args, **full_kwargs)
+
 
 class AsyncExperimentResults:
     def __init__(
@@ -1152,7 +1171,7 @@ class AsyncExperimentResults:
         experiment_manager: _AsyncExperimentManager,
     ):
         self._manager = experiment_manager
-        self._results: List[ExperimentResultRow] = []
+        self._results: list[ExperimentResultRow] = []
         self._lock = asyncio.Lock()
         self._task = asyncio.create_task(self._process_data(self._manager))
         self._processed_count = 0
@@ -1222,6 +1241,7 @@ async def _aforward(
     metadata: dict,
     client: langsmith.Client,
     include_attachments: bool = False,
+    error_handling: Literal["log", "ignore"] = "log",
 ) -> _ForwardResults:
     run: Optional[schemas.RunBase] = None
 
@@ -1229,30 +1249,30 @@ async def _aforward(
         nonlocal run
         run = r
 
+    def _set_reference_example_id(r: rt.RunTree) -> None:
+        r.reference_example_id = example.id
+
+    langsmith_extra = rh.LangSmithExtra(
+        on_end=_get_run,
+        project_name=experiment_name,
+        metadata={
+            **metadata,
+            "example_version": (example.modified_at or example.created_at).isoformat(),
+        },
+        client=client,
+    )
+    if error_handling == "log":
+        langsmith_extra["reference_example_id"] = example.id
+    elif error_handling == "ignore":
+        langsmith_extra["_on_success"] = _set_reference_example_id
+    else:
+        raise ValueError(f"Unrecognized error_handling value: {error_handling=}")
+
     with rh.tracing_context(enabled=True):
         try:
-            args = (
-                (example.inputs, example.attachments)
-                if include_attachments
-                else (example.inputs,)
-            )
-            await fn(
-                *args,
-                langsmith_extra=rh.LangSmithExtra(
-                    reference_example_id=example.id,
-                    on_end=_get_run,
-                    project_name=experiment_name,
-                    metadata={
-                        **metadata,
-                        "example_version": (
-                            example.modified_at.isoformat()
-                            if example.modified_at
-                            else example.created_at.isoformat()
-                        ),
-                    },
-                    client=client,
-                ),
-            )
+            arg_names = _get_target_args(fn)
+            args = [getattr(example, argn) for argn in arg_names]
+            await fn(*args, langsmith_extra=langsmith_extra)
         except Exception as e:
             logger.error(
                 f"Error running target function: {e}", exc_info=True, stacklevel=1
@@ -1320,7 +1340,7 @@ async def async_chain_from_iterable(
 
 
 async def async_iter_from_list(
-    examples: List[schemas.Example],
+    examples: list[schemas.Example],
 ) -> AsyncIterable[schemas.Example]:
     """Convert a list of examples to an async iterable."""
     for example in examples:
